@@ -46,6 +46,7 @@ class CLIError(Exception):
         """
         message += '' if message and message.endswith('\n') else '\n'
         super(CLIError, self).__init__(message)
+        self.message = message
         self.details = (list(details) if (
             isinstance(details, list) or isinstance(details, tuple)) else [
                 '%s' % details]) if details else []
@@ -54,6 +55,9 @@ class CLIError(Exception):
         except ValueError:
             self.importance = 0
 
+    def __str__(self):
+        return self.message
+
 
 class CLIUnimplemented(CLIError):
     def __init__(
@@ -72,23 +76,22 @@ class CLIUnimplemented(CLIError):
 
 
 class CLIBaseUrlError(CLIError):
-    def __init__(self, message='', details=[], importance=2, service=None):
+    def __init__(
+            self,
+            message='', details=[], importance=2, service=None):
         service = '%s' % (service or '')
         message = message or 'No URL for %s' % service.lower()
         details = details or [
-            'Two ways to resolve this:',
-            '(Use the correct cloud name, instead of "default")',
-            'A. (recommended) Let kamaki discover endpoint URLs for all',
-            'services by setting a single Authentication URL and token:',
-            '  /config set cloud.default.url <AUTH_URL>',
-            '  /config set cloud.default.token <t0k3n>',
-            'B. (advanced users) Explicitly set an %s endpoint URL' % (
-                service.upper()),
-            'Note: URL option has a higher priority, so delete it to',
-            'make that work',
-            '  /config delete cloud.default.url',
-            '  /config set cloud.%s.url <%s_URL>' % (
-                service, service.upper())]
+            'To resolve this:',
+            'Set the authentication URL and TOKEN:',
+            '  kamaki config set cloud.<CLOUD NAME>.url <AUTH_URL>',
+            '  kamaki config set cloud.<CLOUD NAME>.token <t0k3n>',
+            'OR',
+            'set a service-specific URL and/or TOKEN',
+            '  kamaki config set '
+            'cloud.<CLOUD NAME>.%s_url <URL>' % (service or '<SERVICE>'),
+            '  kamaki config set '
+            'cloud.<CLOUD NAME>.%s_token <TOKEN>' % (service or '<SERVICE>')]
         super(CLIBaseUrlError, self).__init__(message, details, importance)
 
 
@@ -136,7 +139,11 @@ def raiseCLIError(err, message='', importance=0, details=[]):
     details = list(details) if (
         isinstance(details, list) or isinstance(details, tuple)) else [
             '%s' % details]
-    details += getattr(err, 'details', [])
+    err_details = getattr(err, 'details', [])
+    if isinstance(err_details, list) or isinstance(err_details, tuple):
+        details += list(err_details)
+    else:
+        details.append('%s' % err_details)
 
     origerr = (('%s' % err) or '%s' % type(err)) if err else stack[0]
     message = '%s' % message or origerr
