@@ -1,8 +1,8 @@
 
-__version__ = "0.12.10"
+__version__ = "0.13"
 __version_vcs_info__ = {
     'branch': 'master',
-    'revid': '2632294',
-    'revno': 2178}
+    'revid': 'a4f07c2',
+    'revno': 2473}
 __version_user_email__ = "saxtouri@admin.grnet.gr"
 __version_user_name__ = "Stavros Sachtouris"
